@@ -7,10 +7,10 @@ import (
 	"strings"
 	"sync"
 
-	"golang.org/x/tools/go/analysis"
-
 	_ "github.com/go-critic/go-critic/checkers" // Register go-critic checkers
-	"github.com/go-critic/go-critic/framework/linter"
+	"github.com/go-critic/go-critic/linter"
+
+	"golang.org/x/tools/go/analysis"
 )
 
 type gocritic struct {
@@ -39,38 +39,70 @@ func runAnalyzer(pass *analysis.Pass) (interface{}, error) {
 		return nil, err
 	}
 
+	doneCh := make(chan struct{})
+	diagCh := make(chan analysis.Diagnostic)
+	go func() {
+		for diag := range diagCh {
+			pass.Report(diag)
+		}
+		close(doneCh)
+	}()
+
+	sema := make(chan struct{}, flagConcurrency)
+	var wg sync.WaitGroup
+	wg.Add(len(pass.Files))
+
 	for _, f := range pass.Files {
+		f := f
 		filename := filepath.Base(pass.Fset.Position(f.Pos()).Filename)
 		ctx.SetFileInfo(filename, f)
-		for _, c := range checkers {
-			warnings := c.Check(f)
-			for _, warning := range warnings {
-				diag := analysis.Diagnostic{
-					Pos:     warning.Pos,
-					Message: fmt.Sprintf("%s: %s", c.Info.Name, warning.Text),
+
+		sema <- struct{}{}
+		go func() {
+			defer func() {
+				wg.Done()
+				<-sema
+			}()
+
+			for _, c := range checkers {
+				warnings := c.Check(f)
+				for _, warning := range warnings {
+					diagCh <- asDiag(c, warning)
 				}
-				if warning.HasQuickFix() {
-					diag.SuggestedFixes = []analysis.SuggestedFix{
-						{
-							Message: "suggested replacement",
-							TextEdits: []analysis.TextEdit{
-								{
-									Pos:     warning.Suggestion.From,
-									End:     warning.Suggestion.To,
-									NewText: warning.Suggestion.Replacement,
-								},
-							},
-						},
-					}
-				}
-				pass.Report(diag)
 			}
-		}
+		}()
 	}
 
+	wg.Wait()
+	close(diagCh)
+	<-doneCh
+
 	return nil, nil
 }
 
+func asDiag(c *linter.Checker, warning linter.Warning) analysis.Diagnostic {
+	diag := analysis.Diagnostic{
+		Pos:     warning.Pos,
+		Message: fmt.Sprintf("%s: %s", c.Info.Name, warning.Text),
+	}
+
+	if warning.HasQuickFix() {
+		diag.SuggestedFixes = []analysis.SuggestedFix{
+			{
+				Message: "suggested replacement",
+				TextEdits: []analysis.TextEdit{
+					{
+						Pos:     warning.Suggestion.From,
+						End:     warning.Suggestion.To,
+						NewText: warning.Suggestion.Replacement,
+					},
+				},
+			},
+		}
+	}
+	return diag
+}
+
 // prepareGocritic initializes a new gocririt object,
 // but unlike newGocritic() it could use a cached version.
 func prepareGocritic() (*gocritic, error) {
